/*
Title:			atmega168plot.c
Author:			Windell Oskay  http://www.evilmadscientist.com/
Date Created:   2/2/07
Last Modified:  3/14/07
Copyright 2007 Windell H. Oskay
Distributed under the terms of the GNU General Public License, please see below.


Purpose: Control an analog X-Y Plotter, with pen-lift capability

This code should be relatively straigtforward, so not much documentation is provided.  If you'd like to ask 
questions, suggest improvements, or report success, please use the evilmadscientist forum:
http://www.evilmadscientist.com/forum/

A few quick notes:

Eight digital inputs are used, where a momentary high (button press) 
will activate one of several stored simple programs, e.g., pen lift, move to opposite corners.

A reasonably-good "moveto" routine is provided that will draw straight lines between any two points, 
either fast (if pen up) or slowly and neatly (if pen down).  That, in combination with
simple penup() and pendown() routines can be used for a wide range of graphics.

As a demonstration of those routines, you can use this routine to write short strings, using a simple
vector font included in this file, which uses the moveto(), penup() and pendown() routines to
do its drawing.

You can also use this routine for more complex or algorithmic graphics at your option.


Fusing notes:
We will want to run with the 8 MHz internal RC oscillator,
and with the clock prescaler turned off (divide by 1)

PWM outputs: pins 15,16. (PB1,PB2).  Pen lift: Pin 14.




 This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


*/



#include <avr/io.h> 
#include <avr/pgmspace.h> 
#include <string.h>

//Global Variables:

unsigned int x,y;					//Global variables for present position
unsigned int scalex,scaley;					//Global variables for scale of characters
int xo,yo;					//Global variables for offset of a given character
unsigned int pennstate;				//pen state: 1 is down, 0 is up.


void delay()
{
unsigned int delayvar;
	delayvar = 0; 
	while (delayvar <=  6000U)		
		{ 
			asm("nop");  
			delayvar++;
		}
}


void delayLong()
{
unsigned int delayvar;
	delayvar = 0; 
	while (delayvar <=  60000U)		
		{ 
			asm("nop");  
			delayvar++;
		}
}

void moveto(unsigned int xin, unsigned int yin)
{     

int dx, dy, x0, y0;
unsigned int j;
int dxa,dya;	//Absolute values 
int segments, stepsize,stepsize2;
unsigned int segmentsize;

x0 = x;
y0 = y;

xin /= scalex;  //scaling factors
yin /= scaley;

xin += xo;		//offsets
yin += yo;

dx = xin - x0;
dy = yin - y0;


if (pennstate)  {

if (dy < 0)
	dya = -dy;
else
	dya = dy;
	

if (dx < 0)
	dxa = -dx;
else
	dxa = dx;	

if ((dxa == 0) && dya){		//Travel is strictly in y direction
segmentsize = 4;		

	segments = (dya / segmentsize) - 1;
	
	if (segments > 0)	{
		stepsize = segmentsize;

		if (dy < 0)
			stepsize *= -1;
		
		j = 0; 
		while (j < segments)	
		{ 
			y = y0 + j * stepsize;
			
			
			
			
		if (y <= 1023)
			OCR1B = 1023 - y;
		else
			OCR1B = 0;
			
			
			
			delay();
			j++;
		}
		} 
		
	}

if (dxa && (dya == 0)){		//Travel is strictly in x direction
segmentsize = 4;		

	segments = (dxa / segmentsize) - 1;
	
	if (segments > 0)	{
	stepsize = segmentsize;

	if (dx < 0)
		stepsize *= -1;
		
	j = 0; 
	while (j < segments)	
		{ 
			x = x0 + j * stepsize;
	
		if (x <= 1023)			
			OCR1A = 1023 - x;
		else	
			OCR1A = 0;
			
			
			delay();
			j++;
		}
		} 
		
	}



if (dxa && dya) {		//Travel is in both x and y
segmentsize = 1;		
			
	segments = (dxa / segmentsize) - 1;
	if (segments > 0)	{
	stepsize = segmentsize;
	if (dx < 0)
		stepsize = -segmentsize;
		
	stepsize2 = ( dya << 5 )/segments;
		
	j = 0; 
	while (j < segments)	
		{ 
			x = x0 + j * stepsize;
			
	if (dy > 0)
			y = y0 + ((j * stepsize2)>>5);
	else 
			y = y0 - ((j * stepsize2)>>5);


		if (y <= 1023)
			OCR1B = 1023 - y;
		else
			OCR1B = 0;
		if (x <= 1023)			
			OCR1A = 1023 - x;
		else	
			OCR1A = 0;

			delay();
			delay();

			j++;
		}
		}
		
		x = xin;
		y = yin;
		
		
		if (y <= 1023)
			OCR1B = 1023 - y;
		else
			OCR1B = 0;
		if (x <= 1023)			
			OCR1A = 1023 - x;
		else	
			OCR1A = 0;
			
	}// end	case: if (dxa  && dya)
			
		
		x = x0 + dx;		
		y = y0 + dy;
					
		if (y <= 1023)
			OCR1B = 1023 - y;
		else
			OCR1B = 0;
		if (x <= 1023)			
			OCR1A = 1023 - x;
		else	
			OCR1A = 0;
			
		delayLong();
			
	}			
					
	else		//Pen is not down
	{ 
		
		x = x0 + dx;		
		y = y0 + dy;
										


		if (y <= 1023)
			OCR1B = 1023 - y;
		else
			OCR1B = 0;
		if (x <= 1023)			
			OCR1A = 1023 - x;
		else	
			OCR1A = 0;
			
		delayLong();
		delayLong();
		delayLong();
		delayLong();
		delayLong();
		delayLong();

	}					
							
								
										
	}



void penup()
{     
PORTB = 0U;
delay();
pennstate = 0;
}

void pendown()
{    
PORTB = 1U;
delay();
pennstate = 1;
}






void letterA(){
moveto(0,0);
pendown();
moveto(250,1000); 
moveto(500,0);
penup();
moveto(125,500);
pendown();
moveto(375,500); }

void letterB(){
moveto(0,0);
pendown();
moveto(0,1000);
moveto(400,1000);
moveto(500,900);
moveto(500,600);
moveto(400,500);
moveto(500,400);
moveto(500,100);
moveto(400,0);
moveto(0,0);
penup();
moveto(0,500);
pendown();
moveto(400,500); 

}

void letterC(){
moveto(500,700);
pendown();
moveto(500,800);
moveto(300,1000);
moveto(200,1000);
moveto(0,800);
moveto(0,200);
moveto(200,0);
moveto(300,0);
moveto(500,200); 
}

void letterD(){
moveto(0,0);
pendown();
moveto(0,1000);
moveto(300,1000);
moveto(500,800);
moveto(500,200);
moveto(300,0);
moveto(0,0); 
}

void letterE(){
moveto(500,0);
pendown();
moveto(0,0);
moveto(0,1000);
moveto(500,1000);
penup();
moveto(0,500);
pendown();
moveto(250,500); 
}

void letterF(){

moveto(0,0);
pendown();
moveto(0,1000);
moveto(500,1000);
penup();
moveto(0,500);
pendown();
moveto(250,500); 
}

void letterG(){
moveto(450,400);
pendown();
moveto(500,400);
moveto(500,200);
moveto(300,0);
moveto(200,0);
moveto(0,200);
moveto(0,800);
moveto(200,1000);
moveto(300,1000);
moveto(500,800);
}


void letterH(){

moveto(0,0);
pendown();
moveto(0,1000);
penup();
moveto(500,0);
pendown();
moveto(500,1000);
penup();
moveto(0,500);
pendown();
moveto(500,500);
}





void letterI(){
moveto(0,0);
pendown();
moveto(500,0);
penup();
moveto(250,1000);
pendown();
moveto(250,0);
penup();
moveto(0,1000);
pendown();
moveto(500,1000);
}

void letterJ(){

moveto(500,1000);
pendown();
moveto(125,1000);
penup();
moveto(313,1000);
pendown();
moveto(313,300);
moveto(250,0);
moveto(150,0);
moveto(0,300);
moveto(0,350);
}

void letterK(){

moveto(0,0);
pendown();
moveto(0,1000);
penup();
moveto(0,500);
pendown();
moveto(500,1000);
penup();
moveto(150,650);
pendown();
moveto(475,0);
}


void letterL(){

moveto(0,1000);
pendown();
moveto(0,0);
moveto(500,0);
}

void letterM(){
moveto(0,0);
pendown();
moveto(125,1000);
moveto(250,500);
moveto(375,1000);
moveto(500,0);
}

void letterN(){
moveto(0,0);
pendown();
moveto(0,1000);
moveto(500,0);
moveto(500,1000);
}


void letterO(){
moveto(0,700);
pendown();
moveto(0,300);
moveto(150,0);
moveto(350,0);
moveto(500,300);
moveto(500,700);
moveto(350,1000);
moveto(150,1000);
moveto(0,700);
}

void letterP(){
moveto(0,0);
pendown();
moveto(0,1000);
moveto(400,1000);
moveto(500,900);
moveto(500,600);
moveto(400,500);
moveto(0,500);
}

void letterQ(){
moveto(0,700);
pendown();
moveto(0,300);
moveto(150,0);
moveto(350,0);
moveto(500,300);
moveto(500,700);
moveto(350,1000);
moveto(150,1000);
moveto(0,700);
penup();
moveto(300,300);
pendown();
moveto(500,0); 
}

void letterR(){

moveto(0,0);
pendown();
moveto(0,1000);
moveto(400,1000);
moveto(500,900);
moveto(500,600);
moveto(400,500);
moveto(0,500);
moveto(200,500);
pendown();
moveto(500,0);
}

void letterS(){
moveto(500,850);
pendown();
moveto(500,800);
moveto(350,1000);
moveto(150,1000);
moveto(0,800);
moveto(0,650);
moveto(500,350);
moveto(500,200);
moveto(350,0);
moveto(150,0);
moveto(0,200);
moveto(0,250);
}



void letterT(){
moveto(500,1000);
pendown();
moveto(0,1000);
penup();
moveto(250,1000);
pendown();
moveto(250,0);
}


void letterU(){
moveto(500,1000);
pendown();
moveto(500,0);
penup();
moveto(500,100);
pendown();
moveto(350,0);
moveto(250,0);
moveto(0,100);
moveto(0,1000);
}

void letterV(){
moveto(0,1000);
pendown();
moveto(250,0);
moveto(500,1000);
}



void letterW(){
moveto(0,1000);
pendown();
moveto(125,0);
moveto(250,500);
moveto(375,0);
moveto(500,1000);
}



void letterX(){
moveto(0,0);
pendown();
moveto(500,100);
penup();
moveto(500,0);
pendown();
moveto(0,1000);
}

void letterY(){
moveto(250,0);
pendown();
moveto(250,600);
moveto(0,1000);
penup();
moveto(250,600);
pendown();
moveto(500,1000);
}

void letterZ(){
moveto(0,1000);
pendown();
moveto(500,1000);
moveto(0,0);
moveto(500,0);
}


void exclamation(){
moveto(33,0);
pendown();
moveto(66,0);
moveto(66,100);
moveto(33,100);
moveto(33,0);
penup();
moveto(50,200);
pendown();
moveto(100,1000);
moveto(0,1000);
moveto(50,200);
delay();
}



const char String0[] PROGMEM = "HAPPY BIRTHDAY\rFRANK!";
const char String1[] PROGMEM  = "EVIL MAD\rSCIENTIST";
const char String2[] PROGMEM  = "HELLO MY NAME IS\rALLEN";
const char String3[] PROGMEM  = "SOMEDAY ROBOTS\rWILL RULE THE WORLD"; 

PGM_P StringSet[] PROGMEM = 
{String0,  String1,  String2,  String3 }; 

int main (void)
{ 
PGM_P p;	
char buf[64];				//Set number of characters per string: this eats most of the SRAM!
int writebuf;				// if nonzero, time to write the buffer to the paper.

unsigned int i, stringlength, stringlength2;
unsigned int scalex2,scaley2;


DDRD = 0;		//Port D: All inputs
PORTD = 255U;	//Activate all pull-up resistors.
PORTB = 0U;     // Get ready for pen-up.

TCCR1A = 242;	
TCCR1B = 25;	
ICR1 = 1023;

OCR1A = 1023;			//Start at zero percent duty cycle
OCR1B = 1023;			//Start at zero percent duty cycle

DDRB = _BV(PB0) | _BV(PB1) | _BV(PB2);	/* enable OC1A and OC1B as outputs, sums to #7 decimal. */

x = 0;					//initial position
y = 0;					//initial position

penup();
moveto(0, 0);



scalex = 4;
scaley = 3;

xo = 0;
yo = 500;	



writebuf = 0;
	

for (;;)  // main loop										
{


if ((PIND & 1U) == 0)
penup();

if ((PIND & 2U) == 0)
pendown();

if ((PIND & 4U) == 0){
	xo = 0;
	yo = 0;	
	
	moveto(0, 0);
}

if ((PIND & 8U) == 0) {
	xo = 0;
	yo = 0;	
		
	scalex = 1;
	scaley = 1;

	moveto(1000U, 1000U);
}



// Detect button presses on these Port D lines, use to select a string.
// (This is the lazy way to do it, obviously!)

if ((PIND & 16U) == 0) {
memcpy_P(&p, &StringSet[0], sizeof(PGM_P));
strcpy_P(buf, p);			
writebuf = 1;
}

if ((PIND & 32U) == 0)
{
memcpy_P(&p, &StringSet[1], sizeof(PGM_P));
strcpy_P(buf, p);			
writebuf = 1;

}

if ((PIND & 64U) == 0)
{
memcpy_P(&p, &StringSet[2], sizeof(PGM_P));
strcpy_P(buf, p);			
writebuf = 1;
}

if ((PIND & 128U) == 0)
{
memcpy_P(&p, &StringSet[3], sizeof(PGM_P));
strcpy_P(buf, p);			
writebuf = 1;
}



if (writebuf){

// Estimate total width taken up by string:


i = 0;
stringlength = 0;
stringlength2 = 0;

	while( (buf[i] != '\0') && (buf[i] != '\r')  )		// i.e., while the string has not ended
	{  		
	if ((buf[i] >= 'A') && (buf[i] <= 'Z'))
		stringlength += 600;
	else 
		stringlength += 300;
	i++;
	}
	
	if( buf[i] == '\r' )		// i.e., there IS a return in there
while (buf[i] != '\0') 	
	{  		
	if ((buf[i] >= 'A') && (buf[i] <= 'Z'))
		stringlength2 += 600;
	else 
		stringlength2 += 300;
	i++;
	}
		
	
	
	
	
	
	//Can fit *TWO* standard letters across normal screen, 1000 wide.
	
	i = 0;
	while ((stringlength / i) > 1000 ){
	i++;	
	}
	
	
	scalex = i;
	scaley = (3*scalex)/4;

		xo = 0;	

	i = 0;
	while ((stringlength2 / i) > 1000 ){
	i++;	
	}
	
	scalex2 = i;
	scaley2 = (3*scalex2)/4;

		xo = 10;	

// Character height = 1000/scaley
// With margins of 50 on top and bottom,

yo = 850U - (512U / scaley);




	moveto(xo,yo);
	delayLong();
	
	
	




i = 0;

	while( buf[i] != '\0' )		// i.e., while the string has not ended
	{  		
	
	
	
		if (buf[i] == 'A')
			letterA();
		if (buf[i] == 'B')
			letterB();
		if (buf[i] == 'C')
			letterC();
		if (buf[i] == 'D')
			letterD();
		if (buf[i] == 'E')
			letterE();
		if (buf[i] == 'F')
			letterF();
		if (buf[i] == 'G')
			letterG();
		if (buf[i] == 'H')
			letterH();
						
		if (buf[i] == 'I')
			letterI();
		if (buf[i] == 'J')
			letterJ();
		if (buf[i] == 'K')
			letterK();
		if (buf[i] == 'L')
			letterL();
		if (buf[i] == 'M')
			letterM();
		if (buf[i] == 'N')
			letterN();
		if (buf[i] == 'O')
			letterO();
		if (buf[i] == 'P')
			letterP();

						
		if (buf[i] == 'Q')
			letterQ();
		if (buf[i] == 'R')
			letterR();
		if (buf[i] == 'S')
			letterS();
		if (buf[i] == 'T')
			letterT();
		if (buf[i] == 'U')
			letterU();
		if (buf[i] == 'V')
			letterV();
		if (buf[i] == 'W')
			letterW();
		if (buf[i] == 'X')
			letterX();						
		if (buf[i] == 'Y')
			letterY();
		if (buf[i] == 'Z')
			letterZ();
		if (buf[i] == '!')
			{	exclamation();
				xo -= 300/scalex;	}	
	
																			
		if (buf[i] == ' ')
			xo -= 300/scalex;	 
			
		
		xo += 600/scalex;	
			
				if (buf[i] == '\r')
			{	xo = 10;
				scalex = scalex2;
				scaley = scaley2;
				if ((512U/scaley) < 250U)
					yo = 350U - (512U/scaley);
				else
					yo = 0; 
				moveto(xo,yo);
				delayLong();
			}		
			
			
				penup();	
	
		i++; 
	}

xo = 0;
yo = 0;

scalex = 1;
scaley = 1;

moveto(1000,1000);
penup();

writebuf = 0;

}




}	//End main loop.
	return 0;
}
