/*
Title:			atmega168plot.c  (Transistor version)
Author:			Windell Oskay  http://www.evilmadscientist.com/
Date Created:   2/2/07
Last Modified:  3/14/07
Copyright 2007 Windell H. Oskay
Distributed under the terms of the GNU General Public License, please see below.


Purpose: Control an analog X-Y Plotter, with pen-lift capability

This code should be relatively straigtforward, so not much documentation is provided.  If you'd like to ask 
questions, suggest improvements, or report success, please use the evilmadscientist forum:
http://www.evilmadscientist.com/forum/

A few quick notes:

Eight digital inputs are used, where a momentary high (button press) 
will activate one of several stored simple programs, e.g., pen lift, move to opposite corners.

A reasonably-good "moveto" routine is provided that will draw straight lines between any two points, 
either fast (if pen up) or slowly and neatly (if pen down).  That, in combination with
simple penup() and pendown() routines can be used for a wide range of graphics.

As a demonstration of those routines, plots a simplified photograph of a 2N3904 transistor.
Storing the text as done here is far from the most efficient method. Rather, it is
kept simple for the sake of readability.

You can also use this routine for more complex or algorithmic graphics at your option.


Fusing notes:
We will want to run with the 8 MHz internal RC oscillator,
and with the clock prescaler turned off (divide by 1)

PWM outputs: pins 15,16. (PB1,PB2).  Pen lift: Pin 14.




 This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


*/




#include <avr/io.h> 
#include <avr/pgmspace.h> 
#include <string.h>

//Global Variables:

unsigned int x,y;					//Global variables for present position
unsigned int scalex,scaley;					//Global variables for scale of characters
int xo,yo;					//Global variables for offset of a given character
unsigned int pennstate;				//pen state: 1 is down, 0 is up.


void delay()
{
unsigned int delayvar;
	delayvar = 0; 
	while (delayvar <=  6000U)		
		{ 
			asm("nop");  
			delayvar++;
		}
}


void delayLong()
{
unsigned int delayvar;
	delayvar = 0; 
	while (delayvar <=  60000U)		
		{ 
			asm("nop");  
			delayvar++;
		}
}

void moveto(unsigned int xin, unsigned int yin)
{     

int dx, dy, x0, y0;
unsigned int j;
int dxa,dya;	//Absolute values 
int segments, stepsize,stepsize2;
unsigned int segmentsize;

x0 = x;
y0 = y;

xin /= scalex;  //scaling factors
yin /= scaley;

xin += xo;		//offsets
yin += yo;

dx = xin - x0;
dy = yin - y0;


if (pennstate)  {

if (dy < 0)
	dya = -dy;
else
	dya = dy;
	

if (dx < 0)
	dxa = -dx;
else
	dxa = dx;	

if ((dxa == 0) && dya){		//Travel is strictly in y direction
segmentsize = 4;		

	segments = (dya / segmentsize) - 1;
	
	if (segments > 0)	{
		stepsize = segmentsize;

		if (dy < 0)
			stepsize *= -1;
		
		j = 0; 
		while (j < segments)	
		{ 
			y = y0 + j * stepsize;
			
			
			
			
		if (y <= 1023)
			OCR1B = 1023 - y;
		else
			OCR1B = 0;
			
			
			
			delay();
			j++;
		}
		} 
		
	}

if (dxa && (dya == 0)){		//Travel is strictly in x direction
segmentsize = 4;		

	segments = (dxa / segmentsize) - 1;
	
	if (segments > 0)	{
	stepsize = segmentsize;

	if (dx < 0)
		stepsize *= -1;
		
	j = 0; 
	while (j < segments)	
		{ 
			x = x0 + j * stepsize;
	
		if (x <= 1023)			
			OCR1A = 1023 - x;
		else	
			OCR1A = 0;
			
			
			delay();
			j++;
		}
		} 
		
	}



if (dxa && dya) {		//Travel is in both x and y
segmentsize = 1;		
			
	segments = (dxa / segmentsize) - 1;
	if (segments > 0)	{
	stepsize = segmentsize;
	if (dx < 0)
		stepsize = -segmentsize;
		
	stepsize2 = ( dya << 5 )/segments;
		
	j = 0; 
	while (j < segments)	
		{ 
			x = x0 + j * stepsize;
			
	if (dy > 0)
			y = y0 + ((j * stepsize2)>>5);
	else 
			y = y0 - ((j * stepsize2)>>5);


		if (y <= 1023)
			OCR1B = 1023 - y;
		else
			OCR1B = 0;
		if (x <= 1023)			
			OCR1A = 1023 - x;
		else	
			OCR1A = 0;

			delay();
			delay();

			j++;
		}
		}
		
		x = xin;
		y = yin;
		
		
		if (y <= 1023)
			OCR1B = 1023 - y;
		else
			OCR1B = 0;
		if (x <= 1023)			
			OCR1A = 1023 - x;
		else	
			OCR1A = 0;
			
	}// end	case: if (dxa  && dya)
			
//		x = xin;	
//		y = yin;
		
		x = x0 + dx;		
		y = y0 + dy;
					
		if (y <= 1023)
			OCR1B = 1023 - y;
		else
			OCR1B = 0;
		if (x <= 1023)			
			OCR1A = 1023 - x;
		else	
			OCR1A = 0;
			
		delayLong();
			
	}			
					
	else		//Pen is not down
	{
		//x = xin;	
		//y = yin;
		
		x = x0 + dx;		
		y = y0 + dy;
										


		if (y <= 1023)
			OCR1B = 1023 - y;
		else
			OCR1B = 0;
		if (x <= 1023)			
			OCR1A = 1023 - x;
		else	
			OCR1A = 0;
			
		delayLong();
		delayLong();
		delayLong();
		delayLong();
		delayLong();
		delayLong();

	}					
							
								
										
	}



void penup()
{     
PORTB = 0U;
delay();
pennstate = 0;
}

void pendown()
{    
PORTB = 1U;
delay();
pennstate = 1;
}






void letterA(){
penup();
moveto(430,933);
pendown();
moveto(580,933);
penup();
moveto(430,927);
pendown();
moveto(435,927);
penup();
moveto(575,927);
pendown();
moveto(580,927);
penup();
moveto(430,920);
pendown();
moveto(435,920);
penup();
moveto(575,920);
pendown();
moveto(580,920);
penup();
moveto(430,913);
pendown();
moveto(435,913);
penup();
moveto(575,913);
pendown();
moveto(580,913);
penup();
moveto(430,907);
pendown();
moveto(435,907);
penup();
moveto(575,907);
pendown();
moveto(580,907);
penup();
moveto(430,900);
pendown();
moveto(435,900);
penup();
moveto(575,900);
pendown();
moveto(580,900);
penup();
moveto(430,893);
pendown();
moveto(435,893);
penup();
moveto(575,893);
pendown();
moveto(580,893);
penup();
moveto(430,887);
pendown();
moveto(435,887);
penup();
moveto(460,887);
pendown();
moveto(460,887);
penup();
moveto(480,887);
pendown();
moveto(480,887);
penup();
moveto(510,887);
pendown();
moveto(520,887);
penup();
moveto(575,887);
pendown();
moveto(580,887);
penup();
moveto(430,880);
pendown();
moveto(435,880);
penup();
moveto(460,880);
pendown();
moveto(465,880);
penup();
moveto(475,880);
pendown();
moveto(480,880);
penup();
moveto(505,880);
pendown();
moveto(510,880);
penup();
moveto(520,880);
pendown();
moveto(520,880);
penup();
moveto(535,880);
pendown();
moveto(540,880);
penup();
moveto(555,880);
pendown();
moveto(555,880);
penup();
moveto(575,880);
pendown();
moveto(580,880);
penup();
moveto(430,873);
pendown();
moveto(435,873);
penup();
moveto(455,873);
pendown();
moveto(465,873);
penup();
moveto(475,873);
pendown();
moveto(485,873);
penup();
moveto(520,873);
pendown();
moveto(520,873);
penup();
moveto(540,873);
pendown();
moveto(545,873);
penup();
moveto(555,873);
pendown();
moveto(560,873);
penup();
moveto(575,873);
pendown();
moveto(580,873);
penup();
moveto(430,867);
pendown();
moveto(435,867);
penup();
moveto(455,867);
pendown();
moveto(485,867);
penup();
moveto(515,867);
pendown();
moveto(515,867);
penup();
moveto(540,867);
pendown();
moveto(560,867);
penup();
moveto(575,867);
pendown();
moveto(580,867);
penup();
moveto(430,860);
pendown();
moveto(435,860);
penup();
moveto(450,860);
pendown();
moveto(450,860);
penup();
moveto(470,860);
pendown();
moveto(470,860);
penup();
moveto(485,860);
pendown();
moveto(490,860);
penup();
moveto(510,860);
pendown();
moveto(515,860);
penup();
moveto(540,860);
pendown();
moveto(540,860);
penup();
moveto(550,860);
pendown();
moveto(555,860);
penup();
moveto(575,860);
pendown();
moveto(580,860);
penup();
moveto(430,853);
pendown();
moveto(435,853);
penup();
moveto(450,853);
pendown();
moveto(450,853);
penup();
moveto(490,853);
pendown();
moveto(490,853);
penup();
moveto(505,853);
pendown();
moveto(520,853);
penup();
moveto(535,853);
pendown();
moveto(540,853);
penup();
moveto(550,853);
pendown();
moveto(555,853);
penup();
moveto(575,853);
pendown();
moveto(580,853);
penup();
moveto(430,847);
pendown();
moveto(435,847);
penup();
moveto(575,847);
pendown();
moveto(580,847);
penup();
moveto(430,840);
pendown();
moveto(435,840);
penup();
moveto(575,840);
pendown();
moveto(580,840);
penup();
moveto(430,833);
pendown();
moveto(435,833);
penup();
moveto(450,833);
pendown();
moveto(460,833);
penup();
moveto(485,833);
pendown();
moveto(485,833);
penup();
moveto(515,833);
pendown();
moveto(515,833);
penup();
moveto(525,833);
pendown();
moveto(525,833);
penup();
moveto(555,833);
pendown();
moveto(555,833);
penup();
moveto(575,833);
pendown();
moveto(580,833);
penup();
moveto(430,827);
pendown();
moveto(435,827);
penup();
moveto(450,827);
pendown();
moveto(450,827);
penup();
moveto(460,827);
pendown();
moveto(465,827);
penup();
moveto(480,827);
pendown();
moveto(480,827);
penup();
moveto(495,827);
pendown();
moveto(495,827);
penup();
moveto(510,827);
pendown();
moveto(515,827);
penup();
moveto(525,827);
pendown();
moveto(525,827);
penup();
moveto(545,827);
pendown();
moveto(550,827);
penup();
moveto(575,827);
pendown();
moveto(580,827);
penup();
moveto(430,820);
pendown();
moveto(435,820);
penup();
moveto(460,820);
pendown();
moveto(460,820);
penup();
moveto(480,820);
pendown();
moveto(480,820);
penup();
moveto(495,820);
pendown();
moveto(495,820);
penup();
moveto(510,820);
pendown();
moveto(510,820);
penup();
moveto(525,820);
pendown();
moveto(525,820);
penup();
moveto(545,820);
pendown();
moveto(545,820);
penup();
moveto(555,820);
pendown();
moveto(555,820);
penup();
moveto(575,820);
pendown();
moveto(580,820);
penup();
moveto(430,813);
pendown();
moveto(435,813);
penup();
moveto(460,813);
pendown();
moveto(465,813);
penup();
moveto(485,813);
pendown();
moveto(495,813);
penup();
moveto(510,813);
pendown();
moveto(515,813);
penup();
moveto(525,813);
pendown();
moveto(530,813);
penup();
moveto(540,813);
pendown();
moveto(555,813);
penup();
moveto(575,813);
pendown();
moveto(580,813);
penup();
moveto(430,807);
pendown();
moveto(435,807);
penup();
moveto(460,807);
pendown();
moveto(465,807);
penup();
moveto(490,807);
pendown();
moveto(495,807);
penup();
moveto(510,807);
pendown();
moveto(515,807);
penup();
moveto(525,807);
pendown();
moveto(530,807);
penup();
moveto(540,807);
pendown();
moveto(555,807);
penup();
moveto(575,807);
pendown();
moveto(580,807);
penup();
moveto(430,800);
pendown();
moveto(435,800);
penup();
moveto(450,800);
pendown();
moveto(460,800);
penup();
moveto(485,800);
pendown();
moveto(495,800);
penup();
moveto(515,800);
pendown();
moveto(525,800);
penup();
moveto(555,800);
pendown();
moveto(555,800);
penup();
moveto(575,800);
pendown();
moveto(580,800);
penup();
moveto(430,793);
pendown();
moveto(435,793);
penup();
moveto(575,793);
pendown();
moveto(580,793);
penup();
moveto(430,787);
pendown();
moveto(435,787);
penup();
moveto(520,787);
pendown();
moveto(530,787);
penup();
moveto(575,787);
pendown();
moveto(580,787);
penup();
moveto(430,780);
pendown();
moveto(435,780);
penup();
moveto(450,780);
pendown();
moveto(465,780);
penup();
moveto(480,780);
pendown();
moveto(480,780);
penup();
moveto(490,780);
pendown();
moveto(500,780);
penup();
moveto(515,780);
pendown();
moveto(520,780);
penup();
moveto(530,780);
pendown();
moveto(535,780);
penup();
moveto(545,780);
pendown();
moveto(560,780);
penup();
moveto(575,780);
pendown();
moveto(580,780);
penup();
moveto(430,773);
pendown();
moveto(435,773);
penup();
moveto(450,773);
pendown();
moveto(455,773);
penup();
moveto(480,773);
pendown();
moveto(485,773);
penup();
moveto(500,773);
pendown();
moveto(500,773);
penup();
moveto(515,773);
pendown();
moveto(515,773);
penup();
moveto(545,773);
pendown();
moveto(555,773);
penup();
moveto(575,773);
pendown();
moveto(580,773);
penup();
moveto(430,767);
pendown();
moveto(435,767);
penup();
moveto(450,767);
pendown();
moveto(465,767);
penup();
moveto(480,767);
pendown();
moveto(480,767);
penup();
moveto(490,767);
pendown();
moveto(500,767);
penup();
moveto(515,767);
pendown();
moveto(515,767);
penup();
moveto(545,767);
pendown();
moveto(560,767);
penup();
moveto(575,767);
pendown();
moveto(580,767);
penup();
moveto(430,760);
pendown();
moveto(435,760);
penup();
moveto(450,760);
pendown();
moveto(460,760);
penup();
moveto(480,760);
pendown();
moveto(485,760);
penup();
moveto(495,760);
pendown();
moveto(500,760);
penup();
moveto(515,760);
pendown();
moveto(520,760);
penup();
moveto(535,760);
pendown();
moveto(535,760);
penup();
moveto(545,760);
pendown();
moveto(560,760);
penup();
moveto(575,760);
pendown();
moveto(580,760);
penup();
moveto(430,753);
pendown();
moveto(435,753);
penup();
moveto(450,753);
pendown();
moveto(455,753);
penup();
moveto(480,753);
pendown();
moveto(485,753);
penup();
moveto(495,753);
pendown();
moveto(500,753);
penup();
moveto(520,753);
pendown();
moveto(530,753);
penup();
moveto(545,753);
pendown();
moveto(550,753);
penup();
moveto(560,753);
pendown();
moveto(565,753);
penup();
moveto(575,753);
pendown();
moveto(580,753);
penup();
moveto(430,747);
pendown();
moveto(435,747);
penup();
moveto(450,747);
pendown();
moveto(465,747);
penup();
moveto(480,747);
pendown();
moveto(495,747);
penup();
moveto(530,747);
pendown();
moveto(530,747);
penup();
moveto(565,747);
pendown();
moveto(565,747);
penup();
moveto(575,747);
pendown();
moveto(580,747);
penup();
moveto(430,740);
pendown();
moveto(580,740);
penup();
moveto(430,733);
pendown();
moveto(580,733);
penup();
moveto(430,727);
pendown();
moveto(580,727);
penup();
moveto(435,720);
pendown();
moveto(575,720);
penup();
moveto(450,713);
pendown();
moveto(550,713);
penup();
moveto(455,707);
pendown();
moveto(465,707);
penup();
moveto(475,707);
pendown();
moveto(530,707);
penup();
moveto(550,707);
pendown();
moveto(550,707);
penup();
moveto(455,700);
pendown();
moveto(465,700);
penup();
moveto(480,700);
pendown();
moveto(530,700);
penup();
moveto(550,700);
pendown();
moveto(555,700);
penup();
moveto(455,693);
pendown();
moveto(460,693);
penup();
moveto(485,693);
pendown();
moveto(525,693);
penup();
moveto(550,693);
pendown();
moveto(555,693);
penup();
moveto(455,687);
pendown();
moveto(460,687);
penup();
moveto(480,687);
pendown();
moveto(505,687);
penup();
moveto(515,687);
pendown();
moveto(515,687);
penup();
moveto(550,687);
pendown();
moveto(555,687);
penup();
moveto(450,680);
pendown();
moveto(455,680);
penup();
moveto(485,680);
pendown();
moveto(495,680);
penup();
moveto(505,680);
pendown();
moveto(510,680);
penup();
moveto(555,680);
pendown();
moveto(560,680);
penup();
moveto(450,673);
pendown();
moveto(455,673);
penup();
moveto(500,673);
pendown();
moveto(510,673);
penup();
moveto(555,673);
pendown();
moveto(555,673);
penup();
moveto(445,667);
pendown();
moveto(450,667);
penup();
moveto(505,667);
pendown();
moveto(510,667);
penup();
moveto(440,660);
pendown();
moveto(450,660);
penup();
moveto(505,660);
pendown();
moveto(510,660);
penup();
moveto(560,660);
pendown();
moveto(565,660);
penup();
moveto(440,653);
pendown();
moveto(450,653);
penup();
moveto(500,653);
pendown();
moveto(510,653);
penup();
moveto(560,653);
pendown();
moveto(565,653);
penup();
moveto(435,647);
pendown();
moveto(445,647);
penup();
moveto(500,647);
pendown();
moveto(510,647);
penup();
moveto(560,647);
pendown();
moveto(570,647);
penup();
moveto(435,640);
pendown();
moveto(445,640);
penup();
moveto(500,640);
pendown();
moveto(510,640);
penup();
moveto(565,640);
pendown();
moveto(575,640);
penup();
moveto(435,633);
pendown();
moveto(440,633);
penup();
moveto(500,633);
pendown();
moveto(510,633);
penup();
moveto(565,633);
pendown();
moveto(570,633);
penup();
moveto(430,627);
pendown();
moveto(435,627);
penup();
moveto(505,627);
pendown();
moveto(510,627);
penup();
moveto(570,627);
pendown();
moveto(575,627);
penup();
moveto(430,620);
pendown();
moveto(435,620);
penup();
moveto(505,620);
pendown();
moveto(505,620);
penup();
moveto(570,620);
pendown();
moveto(575,620);
penup();
moveto(425,613);
pendown();
moveto(430,613);
penup();
moveto(505,613);
pendown();
moveto(505,613);
penup();
moveto(575,613);
pendown();
moveto(580,613);
penup();
moveto(420,607);
pendown();
moveto(430,607);
penup();
moveto(505,607);
pendown();
moveto(505,607);
penup();
moveto(580,607);
pendown();
moveto(580,607);
penup();
moveto(415,600);
pendown();
moveto(420,600);
penup();
moveto(505,600);
pendown();
moveto(505,600);
penup();
moveto(580,600);
pendown();
moveto(585,600);
penup();
moveto(410,593);
pendown();
moveto(420,593);
penup();
moveto(510,593);
pendown();
moveto(510,593);
penup();
moveto(585,593);
pendown();
moveto(590,593);
penup();
moveto(405,587);
pendown();
moveto(410,587);
penup();
moveto(505,587);
pendown();
moveto(505,587);
penup();
moveto(590,587);
pendown();
moveto(590,587);
penup();
moveto(395,580);
pendown();
moveto(400,580);
penup();
moveto(505,580);
pendown();
moveto(505,580);
penup();
moveto(595,580);
pendown();
moveto(600,580);
penup();
moveto(390,573);
pendown();
moveto(400,573);
penup();
moveto(505,573);
pendown();
moveto(505,573);
penup();
moveto(605,573);
pendown();
moveto(605,573);
penup();
moveto(380,567);
pendown();
moveto(390,567);
penup();
moveto(505,567);
pendown();
moveto(505,567);
penup();
moveto(605,567);
pendown();
moveto(605,567);
penup();
moveto(370,560);
pendown();
moveto(380,560);
penup();
moveto(505,560);
pendown();
moveto(505,560);
penup();
moveto(615,560);
pendown();
moveto(615,560);
penup();
moveto(365,553);
pendown();
moveto(375,553);
penup();
moveto(505,553);
pendown();
moveto(505,553);
penup();
moveto(620,553);
pendown();
moveto(625,553);
penup();
moveto(355,547);
pendown();
moveto(365,547);
penup();
moveto(505,547);
pendown();
moveto(505,547);
penup();
moveto(625,547);
pendown();
moveto(630,547);
penup();
moveto(345,540);
pendown();
moveto(355,540);
penup();
moveto(505,540);
pendown();
moveto(505,540);
penup();
moveto(630,540);
pendown();
moveto(635,540);
penup();
moveto(335,533);
pendown();
moveto(345,533);
penup();
moveto(505,533);
pendown();
moveto(505,533);
penup();
moveto(640,533);
pendown();
moveto(640,533);
penup();
moveto(325,527);
pendown();
moveto(335,527);
penup();
moveto(505,527);
pendown();
moveto(505,527);
penup();
moveto(645,527);
pendown();
moveto(645,527);
penup();
moveto(320,520);
pendown();
moveto(325,520);
penup();
moveto(505,520);
pendown();
moveto(505,520);
penup();
moveto(650,520);
pendown();
moveto(655,520);
penup();
moveto(310,513);
pendown();
moveto(320,513);
penup();
moveto(505,513);
pendown();
moveto(505,513);
penup();
moveto(655,513);
pendown();
moveto(660,513);
penup();
moveto(300,507);
pendown();
moveto(310,507);
penup();
moveto(505,507);
pendown();
moveto(505,507);
penup();
moveto(665,507);
pendown();
moveto(665,507);
penup();
moveto(290,500);
pendown();
moveto(300,500);
penup();
moveto(505,500);
pendown();
moveto(505,500);
penup();
moveto(670,500);
pendown();
moveto(670,500);
penup();
moveto(280,493);
pendown();
moveto(290,493);
penup();
moveto(505,493);
pendown();
moveto(505,493);
penup();
moveto(675,493);
pendown();
moveto(680,493);
penup();
moveto(270,487);
pendown();
moveto(280,487);
penup();
moveto(505,487);
pendown();
moveto(505,487);
penup();
moveto(685,487);
pendown();
moveto(685,487);
penup();
moveto(260,480);
pendown();
moveto(270,480);
penup();
moveto(505,480);
pendown();
moveto(505,480);
penup();
moveto(690,480);
pendown();
moveto(690,480);
penup();
moveto(250,473);
pendown();
moveto(260,473);
penup();
moveto(505,473);
pendown();
moveto(505,473);
penup();
moveto(695,473);
pendown();
moveto(700,473);
penup();
moveto(240,467);
pendown();
moveto(250,467);
penup();
moveto(505,467);
pendown();
moveto(505,467);
penup();
moveto(705,467);
pendown();
moveto(705,467);
penup();
moveto(230,460);
pendown();
moveto(245,460);
penup();
moveto(505,460);
pendown();
moveto(505,460);
penup();
moveto(710,460);
pendown();
moveto(710,460);
penup();
moveto(225,453);
pendown();
moveto(235,453);
penup();
moveto(505,453);
pendown();
moveto(505,453);
penup();
moveto(715,453);
pendown();
moveto(720,453);
penup();
moveto(215,447);
pendown();
moveto(225,447);
penup();
moveto(505,447);
pendown();
moveto(510,447);
penup();
moveto(720,447);
pendown();
moveto(725,447);
penup();
moveto(205,440);
pendown();
moveto(215,440);
penup();
moveto(505,440);
pendown();
moveto(505,440);
penup();
moveto(730,440);
pendown();
moveto(735,440);
penup();
moveto(195,433);
pendown();
moveto(205,433);
penup();
moveto(505,433);
pendown();
moveto(505,433);
penup();
moveto(735,433);
pendown();
moveto(740,433);
penup();
moveto(185,427);
pendown();
moveto(195,427);
penup();
moveto(505,427);
pendown();
moveto(505,427);
penup();
moveto(745,427);
pendown();
moveto(745,427);
penup();
moveto(175,420);
pendown();
moveto(185,420);
penup();
moveto(505,420);
pendown();
moveto(510,420);
penup();
moveto(750,420);
pendown();
moveto(750,420);
penup();
moveto(170,413);
pendown();
moveto(180,413);
penup();
moveto(505,413);
pendown();
moveto(510,413);
penup();
moveto(755,413);
pendown();
moveto(760,413);
penup();
moveto(160,407);
pendown();
moveto(170,407);
penup();
moveto(505,407);
pendown();
moveto(510,407);
penup();
moveto(765,407);
pendown();
moveto(765,407);
penup();
moveto(150,400);
pendown();
moveto(160,400);
penup();
moveto(505,400);
pendown();
moveto(510,400);
penup();
moveto(770,400);
pendown();
moveto(770,400);
penup();
moveto(140,393);
pendown();
moveto(150,393);
penup();
moveto(505,393);
pendown();
moveto(510,393);
penup();
moveto(775,393);
pendown();
moveto(780,393);
penup();
moveto(130,387);
pendown();
moveto(140,387);
penup();
moveto(505,387);
pendown();
moveto(510,387);
penup();
moveto(780,387);
pendown();
moveto(785,387);
penup();
moveto(120,380);
pendown();
moveto(130,380);
penup();
moveto(505,380);
pendown();
moveto(505,380);
penup();
moveto(790,380);
pendown();
moveto(790,380);
penup();
moveto(110,373);
pendown();
moveto(120,373);
penup();
moveto(505,373);
pendown();
moveto(510,373);
penup();
moveto(795,373);
pendown();
moveto(800,373);
penup();
moveto(100,367);
pendown();
moveto(115,367);
penup();
moveto(505,367);
pendown();
moveto(510,367);
penup();
moveto(800,367);
pendown();
moveto(805,367);
penup();
moveto(95,360);
pendown();
moveto(100,360);
penup();
moveto(505,360);
pendown();
moveto(510,360);
penup();
moveto(805,360);
pendown();
moveto(810,360);
penup();
moveto(75,353);
pendown();
moveto(95,353);
penup();
moveto(505,353);
pendown();
moveto(510,353);
penup();
moveto(815,353);
pendown();
moveto(815,353);
penup();
moveto(75,347);
pendown();
moveto(75,347);
penup();
moveto(85,347);
pendown();
moveto(85,347);
penup();
moveto(505,347);
pendown();
moveto(510,347);
penup();
moveto(820,347);
pendown();
moveto(820,347);
penup();
moveto(505,340);
pendown();
moveto(510,340);
penup();
moveto(830,340);
pendown();
moveto(830,340);
penup();
moveto(505,333);
pendown();
moveto(510,333);
penup();
moveto(835,333);
pendown();
moveto(835,333);
penup();
moveto(505,327);
pendown();
moveto(510,327);
penup();
moveto(840,327);
pendown();
moveto(845,327);
penup();
moveto(505,320);
pendown();
moveto(510,320);
penup();
moveto(845,320);
pendown();
moveto(850,320);
penup();
moveto(505,313);
pendown();
moveto(510,313);
penup();
moveto(850,313);
pendown();
moveto(855,313);
penup();
moveto(505,307);
pendown();
moveto(510,307);
penup();
moveto(860,307);
pendown();
moveto(865,307);
penup();
moveto(510,300);
pendown();
moveto(510,300);
penup();
moveto(865,300);
pendown();
moveto(870,300);
penup();
moveto(505,293);
pendown();
moveto(510,293);
penup();
moveto(870,293);
pendown();
moveto(875,293);
penup();
moveto(505,287);
pendown();
moveto(510,287);
penup();
moveto(880,287);
pendown();
moveto(885,287);
penup();
moveto(505,280);
pendown();
moveto(510,280);
penup();
moveto(885,280);
pendown();
moveto(885,280);
penup();
moveto(505,273);
pendown();
moveto(510,273);
penup();
moveto(890,273);
pendown();
moveto(890,273);
penup();
moveto(510,267);
pendown();
moveto(510,267);
penup();
moveto(505,260);
pendown();
moveto(510,260);
penup();
moveto(505,253);
pendown();
moveto(510,253);
penup();
moveto(505,247);
pendown();
moveto(510,247);
penup();
moveto(505,240);
pendown();
moveto(510,240);
penup();
moveto(505,233);
pendown();
moveto(510,233);
penup();
moveto(505,227);
pendown();
moveto(510,227);
penup();
moveto(505,220);
pendown();
moveto(510,220);
penup();
moveto(505,213);
pendown();
moveto(510,213);
penup();
moveto(505,207);
pendown();
moveto(510,207);
penup();
moveto(505,200);
pendown();
moveto(510,200);
penup();
moveto(505,193);
pendown();
moveto(510,193);
penup();
moveto(505,187);
pendown();
moveto(510,187);
penup();
moveto(510,180);
pendown();
moveto(510,180);
penup();
moveto(505,173);
pendown();
moveto(510,173);
penup();
moveto(510,167);
pendown();
moveto(510,167);
penup();
moveto(510,160);
pendown();
moveto(510,160);
penup();
moveto(505,153);
pendown();
moveto(510,153);
penup();
moveto(510,147);
pendown();
moveto(510,147);
penup();
moveto(510,140);
pendown();
moveto(510,140);
penup();
moveto(510,133);
pendown();
moveto(510,133);
penup();
moveto(510,127);
pendown();
moveto(510,127);
penup();
moveto(505,120);
pendown();
moveto(510,120);
penup();
moveto(505,113);
pendown();
moveto(510,113);
penup();
moveto(510,107);
pendown();
moveto(510,107);
penup();
moveto(505,100);
pendown();
moveto(510,100);
penup();
moveto(505,93);
pendown();
moveto(510,93);
penup();
moveto(510,87);
pendown();
moveto(510,87);
penup();
moveto(505,80);
pendown();
moveto(510,80);
penup();
moveto(505,73);
pendown();
moveto(510,73);
penup();
moveto(505,67);
pendown();
moveto(510,67);
penup();
moveto(505,60);
pendown();
moveto(510,60);
penup();
moveto(1000,7);
 }

const char String0[] PROGMEM = "A";
const char String1[] PROGMEM  = "A";
const char String2[] PROGMEM  = "A";
const char String3[] PROGMEM  = "A"; 

PGM_P StringSet[] PROGMEM = 
{String0,  String1,  String2,  String3 }; 

int main (void)
{ 
PGM_P p;	
char buf[64];				//Set number of characters per string: this eats most of the SRAM!
int writebuf;				// if nonzero, time to write the buffer to the paper.


DDRD = 0;		//Port D: All inputs
PORTD = 255U;	//Activate all pull-up resistors.
PORTB = 0U;     // Get ready for pen-up.

TCCR1A = 242;	
TCCR1B = 25;	
ICR1 = 1023;

OCR1A = 1023;			//Start at zero percent duty cycle
OCR1B = 1023;			//Start at zero percent duty cycle

DDRB = _BV(PB0) | _BV(PB1) | _BV(PB2);	/* enable OC1A and OC1B as outputs, sums to #7 decimal. */

x = 0;					//initial position
y = 0;					//initial position

penup();
moveto(0, 0);



scalex = 4;
scaley = 3;

xo = 0;
yo = 500;	



writebuf = 0;

	/*	
xo = 200;
yo = 200;			
*/

		


	

for (;;)  // main loop										
{


if ((PIND & 1U) == 0)
penup();

if ((PIND & 2U) == 0)
pendown();

if ((PIND & 4U) == 0){
	xo = 0;
	yo = 0;	
	
	moveto(0, 0);
}

if ((PIND & 8U) == 0) {
	xo = 0;
	yo = 0;	
		
	scalex = 1;
	scaley = 1;

	moveto(1000U, 1000U);
}



// Detect button presses on these Port D lines, use to select a string.
// (This is the lazy way to do it, obviously!)

if ((PIND & 16U) == 0) {
memcpy_P(&p, &StringSet[0], sizeof(PGM_P));
strcpy_P(buf, p);			
writebuf = 1;
}

if ((PIND & 32U) == 0)
{
memcpy_P(&p, &StringSet[1], sizeof(PGM_P));
strcpy_P(buf, p);			
writebuf = 1;

}

if ((PIND & 64U) == 0)
{
memcpy_P(&p, &StringSet[2], sizeof(PGM_P));
strcpy_P(buf, p);			
writebuf = 1;
}

if ((PIND & 128U) == 0)
{
memcpy_P(&p, &StringSet[3], sizeof(PGM_P));
strcpy_P(buf, p);			
writebuf = 1;
}



if (writebuf){

	xo = 0;
	yo = 0;	
		
	scalex = 1;
	scaley = 1;

	penup();			
	moveto(0,0);
	delayLong();


	
		letterA();
		penup();	
	

xo = 0;
yo = 0;

scalex = 1;
scaley = 1;

moveto(1000,1000);
penup();

writebuf = 0;

}











}	//End main loop.
	return 0;
}
